@@ -22,12 +22,14 @@
 
     0.0.1 - first release
     0.0.2 - add support for python 3 on Windows and GNU/Linux
-    0.0.3 - remove PNG filters
-          - remove 'ext' argument, using only PNG
-          - do not overwrite existing image files
-          - few optimizations into MSSLinux::get_pixels()
-          - few optimizations into MSSImage::png()
+    0.0.3 - MSSImage: remove PNG filters
+          - MSSImage: remove 'ext' argument, using only PNG
+          - MSSImage: do not overwrite existing image files
+          - MSSImage: few optimizations into png()
+          - MSSLinux: few optimizations into get_pixels()
     0.0.4 - MSSLinux: use of memoization => huge time/operations gains
+    0.0.5 - MSSWindows: few optimizations into _arrange()
+          - MSSImage: code simplified
 
     You can always get the latest version of this module at:
 
@@ -39,7 +41,7 @@
 from __future__ import (unicode_literals, absolute_import,
                         division, print_function)
 
-__version__ = '0.0.4'
+__version__ = '0.0.5'
 __author__ = "Mickaël 'Tiger-222' Schoentgen"
 __copyright__ = '''
     Copyright (c) 2013, Mickaël 'Tiger-222' Schoentgen
@@ -80,29 +82,29 @@ elif system() == 'Linux':
 
     class XWindowAttributes(Structure):
         _fields_ = [
-            ("x",                     c_int32),
-            ("y",                     c_int32),
-            ("width",                 c_int32),
-            ("height",                c_int32),
-            ("border_width",          c_int32),
-            ("depth",                 c_int32),
-            ("visual",                c_ulong),
-            ("root",                  c_ulong),
-            ("class",                 c_int32),
-            ("bit_gravity",           c_int32),
-            ("win_gravity",           c_int32),
-            ("backing_store",         c_int32),
-            ("backing_planes",        c_ulong),
-            ("backing_pixel",         c_ulong),
-            ("save_under",            c_int32),
-            ("colourmap",             c_ulong),
-            ("mapinstalled",          c_uint32),
-            ("map_state",             c_uint32),
-            ("all_event_masks",       c_ulong),
-            ("your_event_mask",       c_ulong),
-            ("do_not_propagate_mask", c_ulong),
-            ("override_redirect",     c_int32),
-            ("screen",                c_ulong)
+            ('x',                     c_int32),
+            ('y',                     c_int32),
+            ('width',                 c_int32),
+            ('height',                c_int32),
+            ('border_width',          c_int32),
+            ('depth',                 c_int32),
+            ('visual',                c_ulong),
+            ('root',                  c_ulong),
+            ('class',                 c_int32),
+            ('bit_gravity',           c_int32),
+            ('win_gravity',           c_int32),
+            ('backing_store',         c_int32),
+            ('backing_planes',        c_ulong),
+            ('backing_pixel',         c_ulong),
+            ('save_under',            c_int32),
+            ('colourmap',             c_ulong),
+            ('mapinstalled',          c_uint32),
+            ('map_state',             c_uint32),
+            ('all_event_masks',       c_ulong),
+            ('your_event_mask',       c_ulong),
+            ('do_not_propagate_mask', c_ulong),
+            ('override_redirect',     c_int32),
+            ('screen',                c_ulong)
         ]
 
     class XImage(Structure):
@@ -295,7 +297,7 @@ class MSSMac(MSS):
     '''
 
     def init(self):
-        ''' Mac initialisations '''
+        ''' Mac OSX initialisations '''
         self.debug('init')
         pass
 
@@ -310,10 +312,10 @@ class MSSMac(MSS):
         if self.oneshot:
             rect = CGRectInfinite
             results.append({
-                b'left'    : int(rect.origin.x),
-                b'top'     : int(rect.origin.y),
-                b'width'   : int(rect.size.width),
-                b'height'  : int(rect.size.height)
+                b'left'  : int(rect.origin.x),
+                b'top'   : int(rect.origin.y),
+                b'width' : int(rect.size.width),
+                b'height': int(rect.size.height)
             })
         else:
             max_displays = 32  # Peut-être augmenté, si besoin...
@@ -531,14 +533,13 @@ class MSSLinux(MSS):
         if image is None:
             raise ValueError('MSSLinux: XGetImage() failed.')
 
-        resultats = {}
-        def pix(pixel):
+        def pix(pixel, _resultats={}):
             ''' Apply shifts to a pixel to get the RGB values.
                 This method uses of memoization.
             '''
-            if not pixel in resultats:
-                resultats[pixel] = b((pixel & 16711680) >> 16) + b((pixel & 65280) >> 8) + b(pixel & 255)
-            return resultats[pixel]
+            if not pixel in _resultats:
+                _resultats[pixel] = b((pixel & 16711680) >> 16) + b((pixel & 65280) >> 8) + b(pixel & 255)
+            return _resultats[pixel]
 
         get_pix = self.XGetPixel
         pixels = [pix(get_pix(image, x, y)) for y in range(height) for x in range(width)]
@@ -698,10 +699,8 @@ class MSSWindows(MSS):
         for y in range(height):
             off = width * (y + 1)
             offset = total - off
-            x = 0
-            while x < width - 2:
+            for x in range(0, width - 2, 3):
                 scanlines[off+x:off+x+3] = b(data[offset+x+2]), b(data[offset+x+1]), b(data[offset+x])
-                x += 3
         return b''.join(scanlines)
 
 
@@ -710,63 +709,50 @@ class MSSImage(object):
     '''
 
     def __init__(self, data=None, width=1, height=1):
-        ''' This method is light and should not change. It is like this
-            to allow the call of extensions() without manipulating
-            real data.
-        '''
-
-        if width < 1 or height < 1:
+        if data is None:
+            raise ValueError('MSSImage: no data to process.')
+        elif width < 1 or height < 1:
             raise ValueError('MSSImage: width or height must be positive.')
 
         self.data = data
         self.width = int(width)
         self.height = int(height)
 
-    def dump(self, output=None):
+    def dump(self, output):
         ''' Dump data to the image file.
-            Returns to created file name if success, else None.
-        '''
-
-        if self.data is None:
-            raise ValueError('MSSImage: no data to process.')
-
-        contents = self.png()
-        with open(output, 'wb') as fileh:
-            fileh.write(contents)
-            return output
-        return None
-
-    def png(self):
-        ''' Pure python PNG implementation.
+            Pure python PNG implementation.
             Image represented as RGB tuples, no interlacing.
             http://inaps.org/journal/comment-fonctionne-le-png
         '''
 
-        to_take = (self.width * 3 + 3) & -4
-        padding = 0 if to_take % 8 == 0 else (to_take % 8) // 2
-        height, data = self.height, self.data
-        scanlines = [b''.join([b'0', data[to_take*y:to_take*y+to_take-padding]]) for y in range(height)]
+        with open(output, 'wb') as fileh:
+            to_take = (self.width * 3 + 3) & -4
+            padding = 0 if to_take % 8 == 0 else (to_take % 8) // 2
+            height, data = self.height, self.data
+            scanlines = [b''.join([b'0', data[to_take*y:to_take*y+to_take-padding]]) for y in range(height)]
 
-        magic = pack(b'>8B', 137, 80, 78, 71, 13, 10, 26, 10)
+            magic = pack(b'>8B', 137, 80, 78, 71, 13, 10, 26, 10)
 
-        # Header: size, marker, data, CRC32
-        ihdr = [b'', b'IHDR', b'', b'']
-        ihdr[2] = pack(b'>2I5B', self.width, self.height, 8, 2, 0, 0, 0)
-        ihdr[3] = pack(b'>I', zlib.crc32(b''.join(ihdr[1:3])) & 0xffffffff)
-        ihdr[0] = pack(b'>I', len(ihdr[2]))
+            # Header: size, marker, data, CRC32
+            ihdr = [b'', b'IHDR', b'', b'']
+            ihdr[2] = pack(b'>2I5B', self.width, self.height, 8, 2, 0, 0, 0)
+            ihdr[3] = pack(b'>I', zlib.crc32(b''.join(ihdr[1:3])) & 0xffffffff)
+            ihdr[0] = pack(b'>I', len(ihdr[2]))
 
-        # Data: size, marker, data, CRC32
-        idat = [b'', b'IDAT', b'', b'']
-        idat[2] = zlib.compress(b''.join(scanlines), 9)
-        idat[3] = pack(b'>I', zlib.crc32(b''.join(idat[1:3])) & 0xffffffff)
-        idat[0] = pack(b'>I', len(idat[2]))
+            # Data: size, marker, data, CRC32
+            idat = [b'', b'IDAT', b'', b'']
+            idat[2] = zlib.compress(b''.join(scanlines), 9)
+            idat[3] = pack(b'>I', zlib.crc32(b''.join(idat[1:3])) & 0xffffffff)
+            idat[0] = pack(b'>I', len(idat[2]))
 
-        # Footer: size, marker, None, CRC32
-        iend = [b'', b'IEND', b'', b'']
-        iend[3] = pack(b'>I', zlib.crc32(iend[1]) & 0xffffffff)
-        iend[0] = pack(b'>I', len(iend[2]))
+            # Footer: size, marker, None, CRC32
+            iend = [b'', b'IEND', b'', b'']
+            iend[3] = pack(b'>I', zlib.crc32(iend[1]) & 0xffffffff)
+            iend[0] = pack(b'>I', len(iend[2]))
 
-        return magic + b''.join(ihdr) + b''.join(idat) + b''.join(iend)
+            fileh.write(magic + b''.join(ihdr) + b''.join(idat) + b''.join(iend))
+            return output
+        return None
 
 
 if __name__ == '__main__':
